# OB - write your own commands.
#
# clock (clk)

"repeater."

# imports

import threading, time

from olib import Object, get_name
from ob.tsk import launch

# defines

def __dir__():
    return ("Repeater", "Timer")

# classes

class Timer(Object):

    "run function at after sleeping nr of seconds."

    def __init__(self, sleep, func, *args, **kwargs):
        super().__init__()
        self.func = func
        self.sleep = sleep
        self.args = args
        self.name = kwargs.get("name", "")
        self.kwargs = kwargs
        self.state = Object()
        self.timer = None

    def run(self, *args, **kwargs):
        "run the timed function"
        self.state.latest = time.time()
        launch(self.func, *self.args, **self.kwargs)

    def start(self):
        "start the timer."
        if not self.name:
            self.name = get_name(self.func)
        timer = threading.Timer(self.sleep, self.run, self.args, self.kwargs)
        timer.setName(self.name)
        timer.setDaemon(True)
        timer.sleep = self.sleep
        timer.state = self.state
        timer.state.starttime = time.time()
        timer.state.latest = time.time()
        timer.func = self.func
        timer.start()
        self.timer = timer
        return timer

    def stop(self):
        "stopt the timer."
        if self.timer:
            self.timer.cancel()

class Repeater(Timer):

    "run function every x seconds."

    def run(self, *args, **kwargs):
        "start a clock after x seconds."
        thr = launch(self.start, **kwargs)
        super().run(*args, **kwargs)
        return thr
