# OB - write your own commands.
#
# handler (hdl)

__version__ = 32

"basic handler and initialisation code."

# imports

import os, sys, threading, time
import olib

from olib import Default, cdir, get_name
from ob.hdl import Handler
from ob.tsk import launch, starttime
from ob.utl import get_exception, root, spl

# defines

def __dir__():
    return ("Cfg", "Kernel", "get_kernel", "kernels", "krn", "starttime", "ver", "wd")

starttime = time.time()

# classes

class Cfg(Default):

    "kernel configuration."

    def __init__(self):
        super().__init__()
        self.users = False

class Kernel(Handler):

    "basic handler."

    def __init__(self):
        super().__init__()
        self.ready = threading.Event()
        self.stopped = False
        self.cfg = Cfg()
        kernels.append(self)

    def announce(self, txt):
        "announce dummy (prevents announcing)."

    def init(self, mns):
        "initialize comma seperated modules list."
        mods = []
        thrs = []
        for mn in spl(mns):
            ms = "ob.%s" % mn
            try:
                mod = self.load_mod(ms)
            except ModuleNotFoundError:
                ms = "obmods.%s" % mn
                try:
                    mod = self.load_mod(ms)
                except ModuleNotFoundError:
                    try:
                        mod = self.load_mod(mn)
                    except ModuleNotFoundError as ex:
                        if mn in str(ex):
                            continue
                        print(get_exception())
                        continue
            mods.append(mod)
            func = getattr(mod, "init", None)
            if func:
                thrs.append(launch(func, self, name=get_name(func)))
        for thr in thrs:
            thr.join()
        return mods

    def scandir(self, name):
        "scan a directory."
        mods = []
        for fn in os.listdir(os.path.join(olib.workdir, name)):
            if fn.startswith("_") or not fn.endswith(".py"):
                continue
            mn = "%s.%s" % (name, fn[:-3])
            module = self.load_mod(mn)
            mods.append(module)
        return mods

    def say(self, channel, txt):
        "print on stdout."
        print(txt)

    def start(self):
        "start the kernel."
        if root():
            assert olib.workdir
            sys.path.insert(0, olib.workdir)
            cdir(olib.workdir)
            cdir(os.path.join(olib.workdir, "obmods", ""))
            self.scandir("obmods")
        super().start()

    def stop(self):
        "stop the kernel."
        self.stopped = True
        self.queue.put(None)

    def wait(self):
        "wait for kernel to finish."
        while not self.stopped:
            time.sleep(60.0)

# runtime

#:
kernels = []

# utilities

def get_kernel():
    "return first registered kernel."
    return kernels[0]

# commands

def krn(event):
    "!krn - kernel dump"
    event.reply(get_kernel())

def ver(event):
    "!ver - show versions."
    k = get_kernel()
    for mod in k.walk("ob,obmods"):
        try:
            event.reply("%s %s" % (mod.__name__, mod.__version__))
        except AttributeError:
            continue

def wd(event):
    "!wd - show workdir"
    event.reply(olib.workdir)
